/* This source is placed in the public domain.  Do what you want with it */


#include <unistd.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <signal.h>
#include <asm/io.h>
#include <config.h>
int aor_return;
unsigned char portdata[10000];

int main(int argc, char* argv[]) {

    static char version[] = "AOR v .01lx.  6-March-97.  Authors: Emarit Ranu and Matthew Quint.";
    int fdSer; 				/* serial file descriptor integer */
    int zero_port_byte;
    struct termios termattribs;
    struct termios *defaults_serial;
    unsigned char decide[22];

    extern outport();
    extern opcode();
    extern readport();
    extern aor();
    void cleanup();
    signal(SIGINT, cleanup);

    aor_return = 1;

    fdSer = port_open(argc, argv);   /* grab the port fd */
    if (fdSer == -1) {
        exit(3);
    }
    tcgetattr(fdSer,defaults_serial);      /* save current serial settings */

    termattribs.c_iflag = IGNBRK | IGNPAR | IMAXBEL | IXOFF;
    termattribs.c_oflag = 0;  
    termattribs.c_cflag = CS8 | CSTOPB | CREAD | CLOCAL;
    termattribs.c_lflag = 0;		    /* Set some term flags */
    
    /*  The ensure there are no read timeouts (possibly writes?) */
    termattribs.c_cc[VMIN] = 1; 
    termattribs.c_cc[VTIME] = 0;
        
    cfsetispeed(&termattribs,B9600);	    /* Set input speed */
    cfsetospeed(&termattribs,B9600);	    /* Set output speed */
    tcsetattr(fdSer,TCSANOW,&termattribs);  /* Set the serial port */

    zero_port_byte = 0x00;

    /*  flush data on port */
    tcflush(fdSer,TCIFLUSH);
    printf("To terminate the program, enter 100.\n");
    while (aor_return == 1) {
       aor(fdSer,decide);
    }
    bye_msg();
}

/*
 *  This function returns the fd for the serial port.
 *  This needs to be rewritten to handle ANY serial port number via user
 *  argument.  Some people may have more than four serial ports.
 */

port_open(int argc, char* argv[]) {
    int fdSer;

    if (argc == 2) {
       switch(argv[1][0]) {
             case '0':
             if ((fdSer = open("/dev/ttyS0", O_RDWR )) < 0) {
                perror("open of /dev/ttyS0 failed!!!\n");
                exit(3);
             }
             break;

             case '1':
             if ((fdSer = open("/dev/ttyS1", O_RDWR )) < 0) {
                perror("open of /dev/ttyS1 failed!!!\n");
                exit(3);
             }
             break;

             case '2':
             if ((fdSer = open("/dev/ttyS2", O_RDWR )) < 0) {
                perror("open of /dev/ttyS2 failed");
                exit(3);
             }
             break;

             case '3':
             if ((fdSer = open("/dev/ttyS3", O_RDWR )) < 0) {
                perror("open of /dev/ttyS3 failed");
                exit(3);
             }
             break;
             default:
             printf("Invalid serial port number!!!\n  Remember:  com1 is 0, com2 is 1, etc.\n");
             return(-1);
       }
    } else {       /* no argument given, spit out usage */
       printf("usage:  aor [port #]\n");
       exit(3);
    }
    return(fdSer);
}

bye_msg() {
     /* generic message to spew when quit or killed */
printf("\n\nThe AOR-AR8000 control program in the making.\n\n");

}

void
cleanup(int fdSer, struct termios *defaults_serial)
{
/* attempts to reset serial port to original state, closes fd
 * as I recall, this never reset the serial port.
 */
  tcsetattr(fdSer, TCSANOW, defaults_serial); 
  tcflush(fdSer,TCIOFLUSH);  /* flush IO data on port */
  close(fdSer);
  bye_msg();
  printf("Killed by signal 2!\n"); 
  exit(3);
}


